@@ -609,7 +609,7 @@ static int ct_setcpus(vps_handler *h, envid_t veid, struct cpu_param *cpu)
 
 	if (cpu->mask)
 		ret = container_apply_config(veid, CPUMASK,
-					     cpumask_bits(cpu->mask));
+					     cpu->mask->bits);
 
 	if (cpu->limit != NULL && *cpu->limit)
 		max_lim = min_ul(*cpu->limit, max_lim);
@@ -843,15 +843,14 @@ static int ct_setcontext(envid_t veid)
 static int ct_chkpnt(vps_handler *h, envid_t veid,
 			const fs_param *fs, int cmd, cpt_param *param)
 {
-	const char *dumpfile = NULL;
+	char dumpfile[PATH_MAX];
 	char statefile[STR_SIZE], buf[STR_SIZE];
 	char *arg[2], *env[4];
 	FILE *sfile;
 	pid_t pid;
 	int ret;
 
-	get_dump_file(veid, param->dumpdir, buf, sizeof(buf));
-	dumpfile = strdup(buf);
+	get_dump_file(veid, param->dumpdir, dumpfile, sizeof(dumpfile));
 
 	arg[0] = SCRIPTDIR "/vps-cpt";
 	arg[1] = NULL;
@@ -891,8 +890,8 @@ static int ct_restore_fn(vps_handler *h, envid_t veid, const vps_res *res,
 			  int wait_p, int old_wait_p, int err_p, void *data)
 {
 	char *argv[2], *env[9];
-	const char *dumpfile = NULL;
-	const char *statefile = NULL;
+	char *dumpfile = NULL;
+	char *statefile = NULL;
 	cpt_param *param = data;
 	veth_dev *veth;
 	char buf[STR_SIZE], *pbuf;
@@ -913,6 +912,8 @@ static int ct_restore_fn(vps_handler *h, envid_t veid, const vps_res *res,
 	env[1] = strdup(buf);
 	snprintf(buf, sizeof(buf), "VE_STATE_FILE=%s", statefile);
 	env[2] = strdup(buf);
+	free(dumpfile);
+	free(statefile);
 
 	pbuf = buf;
 	pbuf += snprintf(buf, sizeof(buf), "VE_VETH_DEVS=");
