@@ -190,6 +190,7 @@ int vzctl_env_switch_snapshot(vps_handler *h, envid_t veid,
 	char fname[PATH_MAX];
 	char snap_xml_tmp[PATH_MAX];
 	char ve_conf_tmp[PATH_MAX] = "";
+	char ve_conf_old[PATH_MAX] = "";
 	char dumpfile[PATH_MAX];
 	char guid_buf[39];
 	const char *guid_tmp = NULL;
@@ -223,6 +224,15 @@ int vzctl_env_switch_snapshot(vps_handler *h, envid_t veid,
 		logger(-1, 0, "Failed to read %s", fname);
 		goto err;
 	}
+
+	vzctl_get_snapshot_dumpfile(fs->private, guid,
+			dumpfile, sizeof(dumpfile));
+
+	if ((param->flags & SNAPSHOT_MUST_RESUME) && stat_file(dumpfile) != 1) {
+		logger(-1, 0, "Error: no dump in snapshot, unable to resume");
+		goto err;
+	}
+
 	logger(0, 0, "Switching to snapshot %s", guid);
 	vzctl_snapshot_tree_set_current(tree, guid);
 	GET_SNAPSHOT_XML_TMP(snap_xml_tmp, fs->private);
@@ -281,24 +291,43 @@ int vzctl_env_switch_snapshot(vps_handler *h, envid_t veid,
 	}
 	if (ve_conf_tmp[0] != '\0') {
 		get_vps_conf_path(veid, fname, sizeof(fname));
+		strncpy(ve_conf_old, fname, sizeof(ve_conf_old - 4));
+		strcat(ve_conf_old, ".old");
+		cp_file(ve_conf_old, fname);
 		if (rename(ve_conf_tmp, fname))
 			logger(-1, errno, "Failed to rename %s -> %s",
 					ve_conf_tmp, fname);
 	}
-	/* resume CT in case dump file exists (no rollback, ignore error) */
-	vzctl_get_snapshot_dumpfile(fs->private, guid, dumpfile, sizeof(dumpfile));
+	/* resume CT in case dump file exists */
 	if (stat_file(dumpfile) == 1 &&	!(param->flags & SNAPSHOT_SKIP_RESUME)) {
-		vps_param *param;
+		vps_param *vps_p;
 
-		param = reread_vps_config(veid);
-		if (param && g_p->res.net.skip_arpdetect)
-			param->res.net.skip_arpdetect =
+		vps_p = reread_vps_config(veid);
+		if (vps_p && g_p->res.net.skip_arpdetect)
+			vps_p->res.net.skip_arpdetect =
 				g_p->res.net.skip_arpdetect;
 		cpt.dumpfile = dumpfile;
-		if (vps_restore(h, veid, (param) ? param : g_p,
-					CMD_RESTORE, &cpt, SKIP_DUMPFILE_UNLINK))
-			logger(-1, 0, "Failed to resume Container");
-		free_vps_param(param);
+		if (vps_restore(h, veid, (vps_p) ? vps_p : g_p,
+					CMD_RESTORE, &cpt, SKIP_DUMPFILE_UNLINK)) {
+			/*
+			 * We have switched to a new snapshot, but
+			 * the restore from a dump file failed.
+			 * Treat it either as a warning or an error,
+			 * depending on the --must-resume flag being set.
+			 */
+			if (param->flags & SNAPSHOT_MUST_RESUME) {
+				logger(-1, 0, "Error: failed to resume CT");
+				free_vps_param(vps_p);
+				goto err4;
+			}
+			else {
+				/* no rollback, ignore restore error */
+				logger(0, 0, "Warning: failed to resume CT, "
+					"ignoring (use --must-resume "
+					"to treat as error)");
+			}
+		}
+		free_vps_param(vps_p);
 	}
 	GET_SNAPSHOT_XML(fname, fs->private);
 	if (rename(snap_xml_tmp, fname))
@@ -313,6 +342,15 @@ int vzctl_env_switch_snapshot(vps_handler *h, envid_t veid,
 	ret = 0;
 	goto out;
 
+err4:
+	/* Restore config */
+	if (ve_conf_old[0] != '\0') {
+		get_vps_conf_path(veid, fname, sizeof(fname));
+		if (rename(ve_conf_old, fname))
+			logger(-1, errno, "Failed to rename %s to %s",
+					ve_conf_old, fname);
+	}
+
 err3:
 	if (guid_tmp != NULL) {
 		switch_param.guid = guid_tmp;
@@ -323,7 +361,7 @@ err3:
 
 err2:
 	if (run && cpt_cmd(h, veid, fs->root, CMD_CHKPNT, CMD_RESUME, 0))
-		logger(-1, 0, "Failed to resume Container on error");
+		logger(-1, 0, "Failed to resume container during rollback");
 
 err1:
 	unlink(snap_xml_tmp);
