@@ -117,6 +117,7 @@ static struct option set_opt[] = {
 	{"cpulimit",	required_argument, NULL, PARAM_CPULIMIT},
 	{"cpus",	required_argument, NULL, PARAM_VCPUS},
 	{"cpumask",	required_argument, NULL, PARAM_CPUMASK},
+	{"nodemask",	required_argument, NULL, PARAM_NODEMASK},
 	/*	create param	*/
 	{"onboot",	required_argument, NULL, PARAM_ONBOOT},
 	{"setmode",	required_argument, NULL, PARAM_SETMODE},
@@ -161,6 +162,15 @@ static struct option set_opt[] = {
 	{NULL, 0, NULL, 0}
 };
 
+#define CHECK_EXTRA_ARGV					\
+if (optind < argc) {						\
+	fprintf(stderr, "Invalid extra arguments: ");		\
+	while (optind < argc)					\
+		fprintf(stderr, "%s ", argv[optind++]);		\
+	fprintf(stderr, "\n");					\
+	return VZ_INVALID_PARAMETER_SYNTAX;			\
+}
+
 static int parse_opt(envid_t veid, int argc, char *argv[], struct option *opt,
 	struct option *internal_opt, vps_param *param)
 {
@@ -211,13 +221,9 @@ static int parse_opt(envid_t veid, int argc, char *argv[], struct option *opt,
 				return VZ_SYSTEM_ERROR;
 		}
 	}
-	if (optind < argc) {
-		printf ("non-option ARGV-elements: ");
-		while (optind < argc)
-			printf ("%s ", argv[optind++]);
-		printf ("\n");
-		return VZ_INVALID_PARAMETER_SYNTAX;
-	}
+
+	CHECK_EXTRA_ARGV
+
 	return 0;
 }
 
@@ -232,6 +238,7 @@ static int parse_startstop_opt(int argc, char **argv, vps_param *param,
 		{"fast", no_argument, NULL, PARAM_FAST},
 		{"skip-umount", no_argument, NULL, PARAM_SKIP_UMOUNT},
 		{"skip-fsck", no_argument, NULL, PARAM_SKIP_FSCK},
+		{"skip-remount", no_argument, NULL, PARAM_SKIP_REMOUNT},
 		{ NULL, 0, NULL, 0 }
 	};
 
@@ -276,12 +283,20 @@ static int parse_startstop_opt(int argc, char **argv, vps_param *param,
 			else
 				goto err;
 			break;
+		case PARAM_SKIP_REMOUNT:
+			if (start)
+				param->opt.skip_remount = YES;
+			else
+				goto err;
+			break;
 		default:
 			ret = VZ_INVALID_PARAMETER_SYNTAX;
 			break;
 		}
 	}
 
+	CHECK_EXTRA_ARGV
+
 	return ret;
 
 err:
@@ -346,6 +361,8 @@ static int start(vps_handler *h, envid_t veid, vps_param *g_p,
 		skip |= SKIP_SETUP;
 	if (cmd_p->opt.skip_fsck == YES)
 		skip |= SKIP_FSCK;
+	if (cmd_p->opt.skip_remount == YES)
+		skip |= SKIP_REMOUNT;
 	/* Try restore first */
 	ret = try_restore(h, veid, g_p, cmd_p, skip);
 	if (ret == 0)
@@ -499,6 +516,7 @@ static int compact(vps_handler *h, envid_t veid, vps_param *g_p,
 	int ret;
 	const char *private	= g_p->res.fs.private;
 	const char *root	= g_p->res.fs.root;
+	char *path = ENV_PATH;
 	char fname[PATH_MAX];
 
 	if (check_var(root, "VE_ROOT is not set"))
@@ -533,6 +551,7 @@ static int compact(vps_handler *h, envid_t veid, vps_param *g_p,
 		return VZ_RESOURCE_ERROR;
 	}
 	logger(1, 0, "Executing %s", cmd);
+	putenv(path);
 	ret = system(cmd);
 	free(cmd);
 
@@ -601,14 +620,13 @@ static int parse_chkpnt_opt(int argc, char **argv, vps_param *vps_p)
 			vps_p->res.net.skip_arpdetect = YES;
 			break;
 		default:
-			if (option_index < 0)
-				logger(-1, 0, "Invalid option -%c", c);
-			else
-				logger(-1, 0, "Invalid option --%s",
-					chkpnt_options[option_index].name);
+			/* Error is printed by getopt_long() */
 			return VZ_INVALID_PARAMETER_SYNTAX;
 		}
 	}
+
+	CHECK_EXTRA_ARGV
+
 	/* Do full checkpointing */
 	if (!cpt->cmd)
 		cpt->cmd = CMD_CHKPNT;
@@ -634,6 +652,7 @@ static int parse_restore_opt(int argc, char **argv, vps_param *vps_p)
 	{"flags",	required_argument, NULL, PARAM_CPU_FLAGS},
 	{"context",	required_argument, NULL, PARAM_CPTCONTEXT},
 	{"skip_arpdetect", no_argument, NULL, PARAM_SKIPARPDETECT},
+	{"skip-remount", no_argument, NULL, PARAM_SKIP_REMOUNT},
 	{ NULL, 0, NULL, 0 }
 	};
 
@@ -670,15 +689,17 @@ static int parse_restore_opt(int argc, char **argv, vps_param *vps_p)
 		case PARAM_SKIPARPDETECT:
 			vps_p->res.net.skip_arpdetect = YES;
 			break;
+		case PARAM_SKIP_REMOUNT:
+			vps_p->opt.skip_remount = YES;
+			break;
 		default:
-			if (option_index < 0)
-				logger(-1, 0, "Invalid option -%c", c);
-			else
-				logger(-1, 0, "Invalid option --%s",
-					restore_options[option_index].name);
+			/* Error is printed by getopt_long() */
 			return VZ_INVALID_PARAMETER_SYNTAX;
 		}
 	}
+
+	CHECK_EXTRA_ARGV
+
 	/* Do full restore */
 	if (!cpt->cmd)
 		cpt->cmd = CMD_RESTORE;
@@ -751,6 +772,7 @@ static int parse_snapshot_switch_opt(envid_t veid, int argc, char **argv,
 	{"skip_arpdetect", no_argument, NULL, PARAM_SKIPARPDETECT},
 	{"skip-resume", no_argument, NULL, PARAM_SNAPSHOT_SKIP_RESUME},
 	{"skip-config", no_argument, NULL, PARAM_SNAPSHOT_SKIP_CONFIG},
+	{"must-resume",	no_argument, NULL, PARAM_SNAPSHOT_MUST_RESUME},
 	{ NULL, 0, NULL, 0 }
 	};
 
@@ -760,9 +782,17 @@ static int parse_snapshot_switch_opt(envid_t veid, int argc, char **argv,
 	ret = parse_opt(veid, argc, argv, opt, snapshot_switch_options, param);
 	free(opt);
 
-	if (ret == 0 && param->snap.guid == NULL)
-		return vzctl_err(VZ_INVALID_PARAMETER_SYNTAX, 0,
-			"Invalid syntax: snapshot id is missing");
+	if (ret == 0) {
+	       if (param->snap.guid == NULL)
+		       return vzctl_err(VZ_INVALID_PARAMETER_SYNTAX, 0,
+				"Invalid syntax: snapshot id is missing");
+
+	       if ((param->snap.flags & SNAPSHOT_SKIP_RESUME) &&
+		   (param->snap.flags & SNAPSHOT_MUST_RESUME))
+		       return vzctl_err(VZ_INVALID_PARAMETER_SYNTAX, 0,
+				"Invalid syntax: --skip-resume and "
+				"--must-resume can not be used together");
+	}
 
 	return ret;
 }
@@ -969,6 +999,17 @@ static int check_set_opt(int argc, char *argv[], vps_param *param)
 					"only makes sense for a ploop CT");
 		}
 	}
+	/* --nodemask requires --cpumask */
+	if (param->res.cpu.nodemask &&
+			!(param->res.cpu.mask || param->res.cpu.cpumask_auto)) {
+		logger(-1, 0, "Error: option --nodemask requires --cpumask");
+		return VZ_INVALID_PARAMETER_SYNTAX;
+	}
+	/* --cpumask auto doesn't make sense without --nodemask */
+	if (param->res.cpu.cpumask_auto && !param->res.cpu.nodemask) {
+		logger(-1, 0, "Error: --cpumask auto requires --nodemask");
+		return VZ_INVALID_PARAMETER_SYNTAX;
+	}
 
 	/* A few options require --save flag */
 	if (!param->opt.save) {
@@ -977,6 +1018,11 @@ static int check_set_opt(int argc, char *argv[], vps_param *param)
 				 " --name option without --save");
 			return VZ_SET_NAME_ERROR;
 		}
+		if (param->res.tmpl.ostmpl) {
+			logger(-1, 0, "Error: unable to use"
+				" --ostemplate option without --save");
+			return VZ_INVALID_PARAMETER_SYNTAX;
+		}
 		if (param->res.misc.stop_timeout >= 0) {
 			logger(-1, 0, "Error: unable to use"
 				" --stop-timeout option without --save");
@@ -1081,7 +1127,6 @@ static int set(vps_handler *h, envid_t veid, vps_param *g_p, vps_param *vps_p,
 {
 	int ret = 0, is_run;
 	dist_actions *actions = NULL;
-	char *dist_name;
 
 	fix_ip_param(g_p, cmd_p);
 	if (!list_empty(&cmd_p->res.veth.dev) ||
@@ -1146,15 +1191,18 @@ static int set(vps_handler *h, envid_t veid, vps_param *g_p, vps_param *vps_p,
 		need_configure(&cmd_p->del_res) ||
 		!list_empty(&cmd_p->res.misc.userpw))
 	{
+		char *dist_name;
+
 		actions = vz_malloc(sizeof(*actions));
 		if (!actions)
 			return VZ_RESOURCE_ERROR;
 		dist_name = get_dist_name(&g_p->res.tmpl);
-		if ((ret = read_dist_actions(dist_name, DIST_DIR, actions))) {
+		ret = read_dist_actions(dist_name, DIST_DIR, actions);
+		free(dist_name);
+		if (ret) {
 			free(actions);
 			return ret;
 		}
-		free(dist_name);
 	}
 	/* Setup password */
 	if (h != NULL && !list_empty(&cmd_p->res.misc.userpw)) {
@@ -1395,6 +1443,7 @@ static int chkpnt(vps_handler *h, envid_t veid, vps_param *g_p, vps_param *cmd_p
 	merge_vps_param(g_p, cmd_p);
 	cmd_p->res.cpt.dumpdir = g_p->res.cpt.dumpdir;
 	if (cmd == CMD_KILL || cmd == CMD_RESUME) {
+		/* FIXME: CRIU support? */
 		ret = cpt_cmd(h, veid, g_p->res.fs.root,
 				CMD_CHKPNT, cmd, cmd_p->res.cpt.ctx);
 		if (ret)
@@ -1425,6 +1474,7 @@ static int restore(vps_handler *h, envid_t veid, vps_param *g_p,
 	vps_param *cmd_p)
 {
 	int cmd;
+	int skip = SKIP_NONE;
 
 	cmd = cmd_p->res.cpt.cmd;
 	merge_vps_param(g_p, cmd_p);
@@ -1432,7 +1482,11 @@ static int restore(vps_handler *h, envid_t veid, vps_param *g_p,
 	if (cmd == CMD_KILL || cmd == CMD_RESUME)
 		return cpt_cmd(h, veid, g_p->res.fs.root,
 				CMD_RESTORE, cmd, cmd_p->res.cpt.ctx);
-	return vps_restore(h, veid, g_p, cmd, &cmd_p->res.cpt, SKIP_NONE);
+
+	if (cmd_p->opt.skip_remount == YES)
+		skip |= SKIP_REMOUNT;
+
+	return vps_restore(h, veid, g_p, cmd, &cmd_p->res.cpt, skip);
 }
 
 static int show_status(vps_handler *h, envid_t veid, vps_param *param)
