@@ -66,14 +66,15 @@ static void usage(int rc)
 "   [--layout ploop|simfs] [--hostname <name>] [--name <name>] [--ipadd <addr>]\n"
 "   [--diskspace <kbytes>] [--diskinodes <NUM> [--private <path>] [--root <path>]\n"
 "   [--local_uid <UID>] [--local_gid <GID>]\n"
-"vzctl start <ctid> [--force] [--wait] [--skip-fsck]\n"
+"vzctl start <ctid> [--force] [--wait] [--skip-fsck] [--skip-remount]\n"
 "vzctl destroy | mount | umount | stop | restart | status <ctid>\n"
 #ifdef HAVE_PLOOP
 "vzctl convert <ctid> [--layout ploop[:mode]]\n"
 "vzctl compact <ctid>\n"
 "vzctl snapshot <ctid> [--id <uuid>] [--name <name>] [--description <desc>]\n"
 "   [--skip-suspend]\n"
-"vzctl snapshot-switch <ctid> --id <uuid> [--skip-resume] [--skip-config]\n"
+"vzctl snapshot-switch <ctid> --id <uuid> [--skip-resume | --must-resume]\n"
+"   [--skip-config]\n"
 "vzctl snapshot-delete <ctid> --id <uuid>\n"
 "vzctl snapshot-mount <ctid> --id <uuid> --target <dir>\n"
 "vzctl snapshot-umount <ctid> --id <uuid>\n"
@@ -91,7 +92,8 @@ static void usage(int rc)
 "   [--nameserver <addr>] [--searchdomain <name>]\n"
 "   [--onboot yes|no] [--bootorder <N>]\n"
 "   [--userpasswd <user>:<passwd>]\n"
-"   [--cpuunits <N>] [--cpulimit <N>] [--cpus <N>] [--cpumask <cpus>]\n"
+"   [--cpuunits <N>] [--cpulimit <N>] [--cpus <N>]\n"
+"   [--cpumask <cpus>] [--nodemask <nodes>]\n"
 "   [--diskspace <soft>[:<hard>]] [--diskinodes <soft>[:<hard>]]\n"
 "   [--quotatime <N>] [--quotaugidlimit <N>] [--mount_opts <opt>[,<opt>...]]\n"
 "   [--offline-resize] [--capability <name>:on|off ...]\n"
@@ -101,7 +103,7 @@ static void usage(int rc)
 "   [--netif_del <ifname>]\n"
 "   [--applyconfig <name>] [--applyconfig_map <name>]\n"
 "   [--features <name:on|off>] [--name <vename>]\n"
-"   [--ioprio <N>] [--iolimit <N>] --iopslimit <N>\n"
+"   [--ioprio <N>] [--iolimit <N>] [--iopslimit <N>]\n"
 "   [--pci_add [<domain>:]<bus>:<slot>.<func>] [--pci_del <d:b:s.f>]\n"
 "   [--iptables <name>] [--disabled <yes|no>]\n"
 "   [--stop-timeout <seconds>\n"
